extern crate clap;

use clap::Parser;

mod codegen;
mod dynconst;
mod env;
mod intrinsics;
mod parser;
mod semant;
mod ssa;
mod types;

use codegen::*;

extern crate hercules_ir;

#[derive(Parser)]
#[command(author, version, about, long_about = None)]
struct Cli {
    src_file : String,
}

fn main() {
    let args = Cli::parse();
    let prg = semant::parse_and_analyze(args.src_file);
    match prg {
        Ok(prg) => {
            let module = codegen_program(prg);

            let mut pm = hercules_opt::pass::PassManager::new(module);
            pm.add_pass(hercules_opt::pass::Pass::Verify);
            pm.add_pass(hercules_opt::pass::Pass::PhiElim);
            pm.add_pass(hercules_opt::pass::Pass::Verify);
            pm.add_pass(hercules_opt::pass::Pass::CCP);
            pm.add_pass(hercules_opt::pass::Pass::DCE);
            pm.add_pass(hercules_opt::pass::Pass::GVN);
            pm.add_pass(hercules_opt::pass::Pass::DCE);
            pm.add_pass(hercules_opt::pass::Pass::Xdot(true));
            pm.add_pass(hercules_opt::pass::Pass::Forkify);
            pm.add_pass(hercules_opt::pass::Pass::DCE);
            pm.add_pass(hercules_opt::pass::Pass::Xdot(true));
            let _ = pm.run_passes();
        },
        Err(errs) => {
            for err in errs{
                eprintln!("{}", err);
            }
        },
    }
}
